package com.simbir.account_sever.common.validate.pagination;

import lombok.RequiredArgsConstructor;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ResponseStatusException;

@Aspect
@Component
@RequiredArgsConstructor
public class PaginationValidator {

    @Value("${base.min-offset}")
    private Integer minOffset;

    @Value("${base.min-limit}")
    private Integer minLimit;

    @Value("${base.max-limit}")
    private Integer maxLimit;

    @Around("@annotation(ValidPagination)")
    public Object validatePagination(ProceedingJoinPoint joinPoint) throws Throwable {
        var signature = (MethodSignature) joinPoint.getSignature();
        var args = joinPoint.getArgs();

        int offset = -1;
        int limit = -1;

        for (int i = 0; i < signature.getMethod().getParameterAnnotations().length; i++) {
            for (var annotation : signature.getMethod().getParameterAnnotations()[i]) {
                if (annotation instanceof PaginationOffset) {
                    offset = (int) args[i];
                } else if (annotation instanceof PaginationLimit) {
                    limit = (int) args[i];
                }
            }
        }

        if (offset < minOffset) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "Invalid from: {0}. Must be greater than or equal to {1}");
        }

        if (limit < minLimit) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "Invalid count: {0}. Must be greater than or equal to {1}");
        }

        if (limit > maxLimit) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "Invalid count: {0}. Must be less than or equal to {1}");
        }

        return joinPoint.proceed();
    }
}
