package com.simbir.account_sever.security.services.activeToken;

import com.simbir.account_sever.security.entities.ActiveTokenEntity;
import com.simbir.account_sever.security.repositories.ActiveTokenRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.server.ResponseStatusException;

import java.util.UUID;

@Service
@RequiredArgsConstructor
public class ActiveTokenServiceImpl implements ActiveTokenService {

    private final ActiveTokenRepository activeTokenRepository;

    @Override
    @Transactional(readOnly = true)
    public ActiveTokenEntity getById(UUID id) {
        return activeTokenRepository.findById(id)
                .orElseThrow(() -> {
                    return new ResponseStatusException(HttpStatus.NOT_FOUND, "Token not found");
                });
    }

    @Override
    @Transactional(readOnly = true)
    public boolean existById(UUID id) {
        return activeTokenRepository.findById(id).isPresent();
    }

    @Override
    @Transactional
    public UUID create() {
        var token = new ActiveTokenEntity();
        return activeTokenRepository.save(token).getId();
    }

    @Override
    @Transactional
    public void deleteById(UUID id) {
        activeTokenRepository.deleteById(id);
    }
}
