package com.simber.payment.common.validate.user;

import com.simber.payment.common.security.context.SecurityContextHolder;
import com.simber.payment.common.security.user.models.UserRole;
import com.simber.payment.user.client.UserServiceClient;
import feign.FeignException;
import jakarta.validation.ConstraintValidator;
import jakarta.validation.ConstraintValidatorContext;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ResponseStatusException;

import java.util.Arrays;
import java.util.Collection;

@Component
@RequiredArgsConstructor
public class UserValidator implements ConstraintValidator<ValidUser, Long> {

    private final UserServiceClient userServiceClient;

    private Collection<UserRole> allowedRoles;
    private boolean requireAll;

    @Override
    public void initialize(ValidUser constraintAnnotation) {
        allowedRoles = Arrays.stream(constraintAnnotation.roles()).toList();
        requireAll = constraintAnnotation.requireAll();
    }

    @Override
    public boolean isValid(Long userId, ConstraintValidatorContext context) {
        try {
            return userServiceClient.exists(userId, allowedRoles, requireAll, SecurityContextHolder.getContext().getAccessToken());
        }catch (FeignException ex) {
            if (ex.status() == -1) {
                throw new ResponseStatusException(HttpStatus.SERVICE_UNAVAILABLE);
            }else {
                return false;
            }
        }
    }
}