package com.simber.purchase.controllers;

import com.simber.purchase.common.security.authenticate.Authenticate;
import com.simber.purchase.common.security.authenticate.models.LogicalOperation;
import com.simber.purchase.common.security.user.UserSession;
import com.simber.purchase.common.security.user.models.UserRole;
import com.simber.purchase.common.security.user.models.UserSessionDetails;
import com.simber.purchase.dto.CancelPurchaseParams;
import com.simber.purchase.dto.CreatePurchaseParams;
import com.simber.purchase.dto.CreateResponseDto;
import com.simber.purchase.dto.PurchaseDto;
import com.simber.purchase.mappers.PurchaseMapper;
import com.simber.purchase.services.PurchaseService;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.util.Collection;
import java.util.Objects;

@RestController
@RequiredArgsConstructor
@RequestMapping("/api/Purchase")
public class PurchaseController {

    private final PurchaseService purchaseService;

    private final PurchaseMapper purchaseMapper;

    @PostMapping
    @Authenticate
    public CreateResponseDto create(@RequestBody CreatePurchaseParams params, @UserSession UserSessionDetails userSessionDetails) {
        var res = purchaseService.create(userSessionDetails.getId(), params);
        return new CreateResponseDto(res.getId(), res.getId());
    }

    @Authenticate
    @PostMapping("/Cancel")
    public void cancel(@RequestBody CancelPurchaseParams params, @UserSession UserSessionDetails userSessionDetails) {
        if (!userSessionDetails.getRoles().contains(UserRole.Admin) && !purchaseService.getById(params.purchaseId()).getUserId().equals(userSessionDetails.getId())) {
            throw new ResponseStatusException(HttpStatus.FORBIDDEN);
        }

        purchaseService.cancel(params);
    }

    @GetMapping("{id}")
    @Authenticate
    public PurchaseDto getById(@PathVariable Long id, @UserSession UserSessionDetails userSessionDetails) {
        if (!userSessionDetails.getRoles().contains(UserRole.Admin) && !purchaseService.getById(id).getUserId().equals(userSessionDetails.getId())) {
            throw new ResponseStatusException(HttpStatus.FORBIDDEN);
        }

        return purchaseMapper.toDto(purchaseService.getById(id));
    }

    @Authenticate
    @GetMapping("/History")
    public Collection<PurchaseDto> getAllHistories(
            @RequestParam(required = false) Long userId,
            @RequestParam(defaultValue = "0") Integer from,
            @RequestParam(defaultValue = "20") Integer count,
            @UserSession UserSessionDetails userSessionDetails
    ) {
        if (!userSessionDetails.getRoles().contains(UserRole.Admin) && !Objects.equals(userId, userSessionDetails.getId())) {
            throw new ResponseStatusException(HttpStatus.FORBIDDEN);
        }

        return purchaseService.getAll(userId, from, count).stream().map(purchaseMapper::toDto).toList();
    }

    @PostMapping("/{paymentId}/Status/Paid")
    public void updateStatusToPaid(@PathVariable Long paymentId) {
        purchaseService.updateStatusToPaid(paymentId);
    }
}
