CREATE EXTENSION IF NOT EXISTS "pgcrypto";

CREATE TABLE IF NOT EXISTS users
(
    id         BIGINT GENERATED ALWAYS AS IDENTITY NOT NULL,
    username   VARCHAR(128)                         NOT NULL,
    password   VARCHAR(256)                        NOT NULL,
    is_deleted BOOLEAN DEFAULT FALSE               NOT NULL,
    CONSTRAINT PK__users__key PRIMARY KEY (id)
    );

CREATE TABLE IF NOT EXISTS user_roles
(
    user_id BIGINT    NOT NULL,
    role_id varchar(16) NOT NULL,
    CONSTRAINT PK__user_roles__key PRIMARY KEY (user_id, role_id)
    );

INSERT INTO users (username, password, is_deleted)
VALUES
    ('user', crypt('user', gen_salt('bf')), FALSE),
    ('seller', crypt('seller', gen_salt('bf')), FALSE),
    ('admin', crypt('admin', gen_salt('bf')), FALSE);

INSERT INTO user_roles (user_id, role_id)
VALUES
    ((SELECT id FROM users WHERE username = 'user'), 'User'),
    ((SELECT id FROM users WHERE username = 'seller'), 'Seller'),
    ((SELECT id FROM users WHERE username = 'admin'), 'Admin');


CREATE TABLE IF NOT EXISTS active_tokens
(
    id UUID NOT NULL DEFAULT gen_random_uuid(),
    CONSTRAINT PK__active_tokens_key PRIMARY KEY (id)
    );

CREATE TABLE IF NOT EXISTS products
(
    id            BIGINT GENERATED ALWAYS AS IDENTITY,
    name          VARCHAR(256) NOT NULL,
	price real NOT NULL,
    description       VARCHAR(512) NOT NULL,
	owner_id BIGINT not null,
	amount int default 0,
    is_deleted    BOOLEAN      NOT NULL DEFAULT FALSE,
    CONSTRAINT PK__products__key PRIMARY KEY (id),
	CONSTRAINT FK__products__product FOREIGN KEY (owner_id) REFERENCES users (id)
    );

CREATE TABLE IF NOT EXISTS product_tags
(
    name        VARCHAR(48) NOT NULL,
    product_id BIGINT      NOT NULL,
    CONSTRAINT PK__product_tags__key PRIMARY KEY (name, product_id),
    CONSTRAINT FK__product_tags__product FOREIGN KEY (product_id) REFERENCES products (id)
    );
	
CREATE TABLE IF NOT EXISTS purchase
(
	id BIGINT GENERATED ALWAYS AS IDENTITY,
	user_id BIGINT not null,
	product_id BIGINT not null,
	amount int not null,
	status varchar(48) not null,
	payment_id BIGINT not null,
	
	 CONSTRAINT PK__purchase__key PRIMARY KEY (id),
	CONSTRAINT FK__purchase__product FOREIGN KEY (product_id) REFERENCES products (id),
	CONSTRAINT FK__purchase__user FOREIGN KEY (user_id) REFERENCES users (id)
);

CREATE TABLE IF NOT EXISTS cards
(
	id BIGINT GENERATED ALWAYS AS IDENTITY,
	number varchar(256) not null,
	name varchar(256) not null,
	date varchar(48) not null,
	cvv varchar(3) not null,
	
	CONSTRAINT PK__cards__key PRIMARY KEY (id)
);

INSERT INTO cards (number, name, date, cvv)
VALUES
    ('0000 0000 0000 0000', 'FOMA KINIAEV', '08/69', '007'),
	('0001 0002 0003 0004', 'DANILA BAGROV', '12/37', '420');
